;; Copyright 2009, 2010, 2014 Kevin Ryde.

;; This file is part of xtide-el.
;;
;; xtide-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; xtide-el is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with xtide-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(require 'cl)
(require 'autoload)

;; (defun my-list-remove-trailing-nils (lst)
;;   (while (and (consp lst)
;;               (eq nil (car (last lst))))
;;     (setq lst (butlast lst 1)))
;;   lst)
  
(defun my-ad-advice-enabled-p (func class name)
  "Return non-nil if FUNC advice CLASS and NAME is enabled."
  (let ((advice (ad-find-advice func class name)))
    (and advice
         (ad-advice-enabled advice))))

(defmacro my-with-ad-enable-advice (func class name &rest body)
  "Evaluate BODY with FUNC advice CLASS and NAME temporarily enabled.
And `unwind-protect' ensures the advice is re-disabled (if it was
disabled to start with) no matter what BODY does.

In the current implementation CLASS must be a specific class,
like `before' or `around', it can't be `any'."

  `(let ((my-with-ad-enable-advice--old
          (my-ad-advice-enabled-p ,func ,class ,name)))
     (unwind-protect
         (progn
           (unless my-with-ad-enable-advice--old
             (ad-enable-advice ,func ,class ,name)
             (ad-activate ,func))
           ,@body)
       (unless my-with-ad-enable-advice--old
         (ad-disable-advice ,func ,class ,name)
         (ad-activate ,func)))))
(put 'my-with-ad-enable-advice 'lisp-indent-function 3)

(defun my-string-first-line (str)
  "Return the first line of STR.
This means everything up to but not including the first newline,
or the whole string if no newlines at all."
  (if (string-match "\n" str)
      (substring str 0 (match-beginning 0))
    str))
;; (my-string-first-line "abc\ndef\nghi")
;; (my-string-first-line "abc")

(defun my-oneline-autoload-docstring (form)
  "If FORM is an (autoload ...) return it with a one line docstring.
Otherwise return FORM unchanged.

The idea here is to keep lots of verbiage out of autoloads,
leaving it for the full definition once loaded.  For
byte-compiled autoloads there's no need for this, as the dynamic
docstrings defer them until needed, but for plain .el a shorter
docstring may save a few bytes of core."

  ;;  0        1     2      3           4                5
  ;; (autoload 'func "file" "docstring" interactive-flag type)
  ;;
  (when (and (listp form)
             (eq 'autoload (car form))
             (stringp (nth 3 form)))
    (setq form (copy-sequence form))
    (setf (nth 3 form) (my-string-first-line (nth 3 form))))
  form)

;; (my-oneline-autoload-docstring nil)
;; (my-oneline-autoload-docstring '(autoload 'func "file"))
;; (my-oneline-autoload-docstring '(autoload 'func "file" "one\ntwo" interactive-flag))

(defadvice make-autoload (after my-oneline-docstring disable)
  "Truncate to one line docstrings using `my-oneline-autoload-docstring'."
  (setq ad-return-value (my-oneline-autoload-docstring ad-return-value)))

(defun my-generate-file-autoloads (filename)
  "Like `generate-file-autoloads' but with one-line docstrings."
  (let ((print-quoted t))
    (my-with-ad-enable-advice 'make-autoload 'after 'my-oneline-docstring
      (generate-file-autoloads filename))))

;; some nonsense in emacs24 forces the generate to be in a file-visiting
;; buffer, and in the same directory as xtide.el or else the autoloads
;; include a path "../xtide"
(insert (progn
          (find-file "../my-generate-autoloads.out.el")
          (erase-buffer)
          (my-generate-file-autoloads "xtide.el")
          (prog1 (buffer-string)
            (set-buffer-modified-p nil)
            (kill-buffer (current-buffer)))))
