;; Copyright 2007, 2008, 2009, 2010, 2012, 2013, 2014, 2015, 2019 Kevin Ryde   -*- coding: latin-1 -*-

;; This file is part of xtide-el.
;;
;; xtide-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; xtide-el is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
;; FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
;; more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with xtide-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.


(let ((dir (expand-file-name "..")))
  (message "load directory %S" dir)
  (setq load-path (cons dir load-path)))

;; the base debian "xtide" package /usr/share/xtide/harmonics-initial.tcd
;; has only one location "Alameda, San Francisco Bay, California"
(defconst my-xtide-test-location
  "Alameda, San Francisco Bay, California")
(message "my-xtide-test-location is %S" my-xtide-test-location)

(defvar my-xtide-available-p
  (let ((process-environment process-environment))
    (setenv "DISPLAY" nil)
    (equal 0 (let ((process-connection-type nil)) ;; pipe
               (call-process "tide" nil nil nil
                             "-ma" "-l" my-xtide-test-location)))))
(message "my-xtide-available-p is %S" my-xtide-available-p)

(defun my-run-test ()

  (let ((count 0))

    ;;-------------------------------------------------------------------------
    ;; autoloads

    ;; emacs24 `generate-file-autoloads' wants to be in a file-visiting
    ;; buffer
    (require 'autoload)
    (find-file "t-xtide--generated-loaddefs.el")
    (erase-buffer)
    (generate-file-autoloads "../xtide.el")
    (eval-buffer)
    (save-buffer)
    (kill-buffer (current-buffer))
    (delete-file "t-xtide--generated-loaddefs.el")

    (mapcar (lambda (func)
              (unless (fboundp func)
                (error "`%S' not autoloaded" func))
              (setq count (1+ count)))
            '(xtide
              xtide-calendar-setups
              xtide-location-bookmark-jump))

    (mapcar (lambda (elem)
              (let* ((hook (nth 0 elem))
                     (func (nth 1 elem))
                     (options (get hook 'custom-options)))
                (unless (and (listp options)
                             (memq func options))
                  (error "`%S' not an option on `%S'" func hook)))
              (setq count (1+ count)))

            '((calendar-load-hook xtide-calendar-setups)))


    ;;-------------------------------------------------------------------------
    ;; safe-local-variable-p

    (if (not (fboundp 'safe-local-variable-p))
        (message "skip, no `safe-local-variable-p'")

      (unless (safe-local-variable-p 'xtide-location "Some Where")
        (error "xtide-location=string not safe-local-variable-p"))
      (setq count (1+ count))

      (when (safe-local-variable-p 'xtide-location 123)
        (error "xtide-location=123 is safe-local-variable-p"))
      (setq count (1+ count)))


    ;;-------------------------------------------------------------------------
    ;; risky-local-variable-p

    (if (not (fboundp 'risky-local-variable-p))
        (message "skip, no `risky-local-variable-p'")

      (unless (risky-local-variable-p 'xtide-run-switches '("-foo"))
        (error "xtide-run-switches not `risky-local-variable-p'"))
      (setq count (1+ count)))


    ;;-------------------------------------------------------------------------
    ;; byte compile state

    (require 'xtide)
    (message "`xtide' is byte-code %S"
             (byte-code-function-p (symbol-function 'xtide)))


    ;;-------------------------------------------------------------------------
    ;; customize

    (save-excursion
      (customize-group 'xtide)
      (kill-buffer nil))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; `custom-group-of-mode' for `customize-mode'

    (if (not (fboundp 'custom-group-of-mode))
        (message "skip, no `custom-group-of-mode' in this emacs")

      (let* ((mode 'xtide-mode)
             (want 'xtide)
             (got  (custom-group-of-mode mode)))
        (unless (equal got want)
          (error "custom-group-of-mode for %S is %S expected %S"
                 mode got want))
        (setq count (1+ count)))

      (let* ((mode 'xtide-location-mode)
             (want 'xtide)
             (got  (custom-group-of-mode mode)))
        (unless (equal got want)
          (error "custom-group-of-mode for %S is %S expected %S"
                 mode got want))
        (setq count (1+ count))))

    ;;-------------------------------------------------------------------------
    ;; xtide-with-TZ
    ;; note no `t' for GMT in xemacs21, hence "GMT+0"

    (require 'xtide)
    (if (not (fboundp 'xtide-with-TZ))
        (message "skip, no xtide-with-TZ when running byte compiled")

      (let ((time (current-time))
            gmt-zone cst-zone wst-zone)
        (set-time-zone-rule "GMT+0")
        (setq gmt-zone (current-time-zone time))
        (setenv "TZ" "WST-8")
        (setq wst-zone (current-time-zone time))
        (setenv "TZ" "CST-9")
        (setq cst-zone (current-time-zone time))

        (let (got-zone)
          (xtide-with-TZ "GMT+0"
            (setq got-zone (current-time-zone time)))
          (or (equal got-zone gmt-zone)
              (error "xtide-with-TZ didn't apply GMT (plain)")))
        (setq got-zone (current-time-zone time))
        (or (equal got-zone cst-zone)
            (error "xtide-with-TZ didn't restore EST"))
        (setq count (1+ count))

        (xtide-with-TZ "WST-8"
          (xtide-with-TZ "GMT+0"
            (or (equal gmt-zone (current-time-zone time))
                (error "xtide-with-TZ didn't apply GMT (nested)")))
          (let ((got-zone (current-time-zone time)))
            (or (equal wst-zone got-zone)
                (error "xtide-with-TZ didn't restore to WST: got %S want %S"
                       got-zone wst-zone))))
        (or (equal got-zone cst-zone)
            (error "xtide-with-TZ didn't restore to EST"))
        (setq count (1+ count))))

    ;;-------------------------------------------------------------------------
    ;; xtide-location-sort-alphabetical

    (require 'xtide)
    (progn
      (switch-to-buffer "*t-xtide.el -- temp*")
      (mapcar (lambda (elem)
                (let ((always (nth 0 elem))
                      (strs   (nth 1 elem))
                      (want   (nth 2 elem)))

                  (if (not (or always
                               (or (fboundp 'put-case-table)     ;; xemacs21
                                   (fboundp 'with-case-table)))) ;; emacs22 up
                      (message "skip no `with-case-table'")

                    (erase-buffer)
                    (mapcar (lambda (str)
                              (insert str "                 Sub 0.0000 N, 0.0000 W\n"))
                            strs)

                    (xtide-location-sort-alphabetical)
                    (let ((got (mapcar (lambda (line)
                                         (substring line 0 1))
                                       (delete "" (split-string (buffer-string)
                                                                "\n")))))
                      (unless (equal got want)
                        (error "xtide-location-sort-alphabetical wrong\n  got %S want %S at count %d %s"
                               got want count elem))))
                  (setq count (1+ count))))

              '((t   ("e" "f" "d") ("d" "e" "f"))
                (t   ("e" "f" "d") ("d" "e" "f"))

                (nil ("" "f" "a" "d" "z") ("a" "d" "" "f" "z"))
                (nil ("" "f" "d") ("d" "" "f"))
                ))

      (kill-buffer "*t-xtide.el -- temp*"))

    (progn
      (switch-to-buffer "*t-xtide.el -- temp*")
      (erase-buffer)
      (insert "C\nB\nA\n")
      (sort-lines nil (point-min) (point-max))
      (let ((want "A\nB\nC\n")
            (got  (buffer-string)))
        (unless (equal got want)
          (error "subsequent sort-lines wrong\n  got %S want %S" got want)))
      (setq count (1+ count)))

    ;;-------------------------------------------------------------------------
    ;; xtide-location-line-lat-long

    (mapcar (lambda (elem)
              (let* ((line      (nth 0 elem))
                     (want      (nth 1 elem))
                     (want-lat  (car  want))
                     (want-long (cadr want)))
                (let* ((got      (xtide-location-line-lat-long line))
                       (got-lat  (car  got))
                       (got-long (cadr got))
                       (diff-lat  (abs (- got-lat  want-lat)))
                       (diff-long (abs (- got-long want-long))))
                  (unless (and (< diff-lat  1e-4)
                               (< diff-long 1e-4))
                    (error "xtide-location-line-lat-long\ngot  %s\nwant %s\ndiffs %s %s\nat elem %d %s"
                           got want diff-lat diff-long count elem))))
              (setq count (1+ count)))
            '(
              ;; NSEW signs
              ("Som, wher  Ref 33.9833 N, 151.2167 E" ( 0.593120  2.639229))
              ("Som, wher  Ref 33.9833 N, 151.2167 W" ( 0.593120 -2.639229))
              ("Som, wher  Ref 33.9833 S, 151.2167 E" (-0.593120  2.639229))
              ("Som, wher  Ref 33.9833 S, 151.2167 W" (-0.593120 -2.639229))))

    ;;-------------------------------------------------------------------------
    ;; xtide-show-about-location

    ;; see that buttonizing works
    (when my-xtide-available-p
      (setq xtide-location my-xtide-test-location)
      (xtide)
      (xtide-show-about-location)
      (kill-buffer nil)
      (setq count (1+ count)))

    ;;-------------------------------------------------------------------------
    ;; xtide-location-sort-locality-key

    (switch-to-buffer "*t-xtide.el -- temp*")
    (mapcar (lambda (elem)
              (let ((str  (nth 0 elem))
                    (want (nth 1 elem)))
                (erase-buffer)
                (insert str "                 Sub 0.0000 N, 0.0000 W\n")
                (goto-char (point-min))
                (let ((got (xtide-location-sort-locality-key)))
                  (unless (equal got want)
                    (error "xtide-location-sort-locality-key: got '%s' want '%s' at elem %d %s"
                           got want count elem))))
              (setq count (1+ count)))

            '(("Foo" "foo")
              ("Foo, Bar" "bar,foo")
              ("Foo, Bar (2)" "bar,foo,(2)")
              ("Foo, Bar (2)" "bar,foo,(2)")
              ("Foo, Bar (2) (sub)" "bar,foo,(2) (sub)")
              ("Foo, Bar - READ this" "bar,foo,- read this")))
    (kill-buffer "*t-xtide.el -- temp*")


    ;;-------------------------------------------------------------------------
    ;; xtide-location-value

    (mapcar (lambda (elem)
              (setq xtide-location             (nth 0 elem))
              (setenv "XTIDE_DEFAULT_LOCATION" (nth 1 elem))
              (let ((want (nth 2 elem))
                    (got  (xtide-location-value)))
                (unless (equal got want)
                  (error "xtide-location-value: got '%s' want '%s' at elem %d %s"
                         got want count elem)))
              (setq count (1+ count)))

            '((nil nil nil)
              ("" nil nil)
              (" " nil nil)
              ("\n" nil nil)
              (nil "" nil)
              (nil " " nil)
              (nil "\n" nil)
              ("Foo bar"   "" "Foo bar")
              (" Foo bar"  "" "Foo bar")
              ("Foo bar\t" "" "Foo bar")
              (""   "Foo bar"   "Foo bar")
              ("\n" "Foo bar"   "Foo bar")
              (nil  "Foo bar\t" "Foo bar")
              ))

    ;;-------------------------------------------------------------------------
    ;; bookmarks

    (require 'bookmark)
    (if (not (boundp 'bookmark-make-record-function))
        ;; emacs23 up
        (message "skip no `bookmark-make-record-function' in this bookmark.el")

      (let ((bmk "t-xtide.el test"))
        (message "bookmark temporary: %S" bmk)

        (xtide-location-select)
        (goto-char (point-min))
        (forward-line 10)
        (bookmark-set "t-xtide.el test")
        (kill-buffer (current-buffer))
        (bookmark-jump bmk)
        (unless (= 10 (count-lines (point-min) (point)))
          (error "bookmark-jump wrong place"))
        (setq count (1+ count))

        (forward-line 20)
        (bookmark-jump bmk)
        (unless (= 10 (count-lines (point-min) (point)))
          (error "bookmark-jump wrong place"))

        (bookmark-delete bmk)
        (bookmark-delete bmk)
        (bookmark-delete bmk)))


    ;;-------------------------------------------------------------------------
    ;; unload

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)
    (message "my-unload-feature-function-p is %S" my-unload-feature-function-p)

    (if (not my-unload-feature-function-p)
        (message "skip, no FEATURE-unload-function")

      (setq xtide-location my-xtide-test-location)
      (xtide)
      (xtide-location-select)
      (unload-feature 'xtide t)

      ;; no longer kill-buffer on unload
      ;; (when (get-buffer "*xtide*")
      ;;   (error "buffer *xtide* left on unload"))
      ;; (when (get-buffer "*xtide-location*")
      ;;   (error "buffer *xtide-location* left on unload"))
      ;; (setq count (1+ count))
      )

    ;;-------------------------------------------------------------------------
    ;; customize

    (save-excursion
      (customize-group 'xtide)
      (kill-buffer nil))


    ;;-------------------------------------------------------------------------
    (message "  %d tests" count)
    count))
